from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class TenantUserResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The email property
    email: Optional[str] = None
    # The fullName property
    full_name: Optional[str] = None
    # The joinedAt property
    joined_at: Optional[datetime.datetime] = None
    # The membershipId property
    membership_id: Optional[str] = None
    # The role property
    role: Optional[str] = None
    # The status property
    status: Optional[str] = None
    # The userId property
    user_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TenantUserResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TenantUserResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TenantUserResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "email": lambda n : setattr(self, 'email', n.get_str_value()),
            "fullName": lambda n : setattr(self, 'full_name', n.get_str_value()),
            "joinedAt": lambda n : setattr(self, 'joined_at', n.get_datetime_value()),
            "membershipId": lambda n : setattr(self, 'membership_id', n.get_str_value()),
            "role": lambda n : setattr(self, 'role', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "userId": lambda n : setattr(self, 'user_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("email", self.email)
        writer.write_str_value("fullName", self.full_name)
        writer.write_datetime_value("joinedAt", self.joined_at)
        writer.write_str_value("membershipId", self.membership_id)
        writer.write_str_value("role", self.role)
        writer.write_str_value("status", self.status)
        writer.write_str_value("userId", self.user_id)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .emergency_kit_instructions import EmergencyKitInstructions

@dataclass
class EmergencyKit(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The generatedAt property
    generated_at: Optional[str] = None
    # The instructions property
    instructions: Optional[EmergencyKitInstructions] = None
    # The masterKeyFingerprint property
    master_key_fingerprint: Optional[str] = None
    # The recoveryKeyFingerprint property
    recovery_key_fingerprint: Optional[str] = None
    # The tenantId property
    tenant_id: Optional[str] = None
    # The tenantSlug property
    tenant_slug: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> EmergencyKit:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: EmergencyKit
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return EmergencyKit()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .emergency_kit_instructions import EmergencyKitInstructions

        from .emergency_kit_instructions import EmergencyKitInstructions

        fields: dict[str, Callable[[Any], None]] = {
            "generatedAt": lambda n : setattr(self, 'generated_at', n.get_str_value()),
            "instructions": lambda n : setattr(self, 'instructions', n.get_object_value(EmergencyKitInstructions)),
            "masterKeyFingerprint": lambda n : setattr(self, 'master_key_fingerprint', n.get_str_value()),
            "recoveryKeyFingerprint": lambda n : setattr(self, 'recovery_key_fingerprint', n.get_str_value()),
            "tenantId": lambda n : setattr(self, 'tenant_id', n.get_str_value()),
            "tenantSlug": lambda n : setattr(self, 'tenant_slug', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("generatedAt", self.generated_at)
        writer.write_object_value("instructions", self.instructions)
        writer.write_str_value("masterKeyFingerprint", self.master_key_fingerprint)
        writer.write_str_value("recoveryKeyFingerprint", self.recovery_key_fingerprint)
        writer.write_str_value("tenantId", self.tenant_id)
        writer.write_str_value("tenantSlug", self.tenant_slug)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class AuthDiagnostics(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The configuredKeycloakUrl property
    configured_keycloak_url: Optional[str] = None
    # The expectedIssuer property
    expected_issuer: Optional[str] = None
    # The hasAuthorizationHeader property
    has_authorization_header: Optional[bool] = None
    # The isAuthenticated property
    is_authenticated: Optional[bool] = None
    # The tokenAudience property
    token_audience: Optional[str] = None
    # The tokenClaims property
    token_claims: Optional[list[str]] = None
    # The tokenIssuer property
    token_issuer: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AuthDiagnostics:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AuthDiagnostics
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AuthDiagnostics()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "configuredKeycloakUrl": lambda n : setattr(self, 'configured_keycloak_url', n.get_str_value()),
            "expectedIssuer": lambda n : setattr(self, 'expected_issuer', n.get_str_value()),
            "hasAuthorizationHeader": lambda n : setattr(self, 'has_authorization_header', n.get_bool_value()),
            "isAuthenticated": lambda n : setattr(self, 'is_authenticated', n.get_bool_value()),
            "tokenAudience": lambda n : setattr(self, 'token_audience', n.get_str_value()),
            "tokenClaims": lambda n : setattr(self, 'token_claims', n.get_collection_of_primitive_values(str)),
            "tokenIssuer": lambda n : setattr(self, 'token_issuer', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("configuredKeycloakUrl", self.configured_keycloak_url)
        writer.write_str_value("expectedIssuer", self.expected_issuer)
        writer.write_bool_value("hasAuthorizationHeader", self.has_authorization_header)
        writer.write_bool_value("isAuthenticated", self.is_authenticated)
        writer.write_str_value("tokenAudience", self.token_audience)
        writer.write_collection_of_primitive_values("tokenClaims", self.token_claims)
        writer.write_str_value("tokenIssuer", self.token_issuer)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .handshake.handshake_request_builder import HandshakeRequestBuilder

class UsersRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api/users
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new UsersRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api/users", path_parameters)
    
    @property
    def handshake(self) -> HandshakeRequestBuilder:
        """
        The handshake property
        """
        from .handshake.handshake_request_builder import HandshakeRequestBuilder

        return HandshakeRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class RotatorFieldDefDto(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The helpText property
    help_text: Optional[str] = None
    # The key property
    key: Optional[str] = None
    # The label property
    label: Optional[str] = None
    # The placeholder property
    placeholder: Optional[str] = None
    # The required property
    required: Optional[bool] = None
    # The type property
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RotatorFieldDefDto:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RotatorFieldDefDto
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RotatorFieldDefDto()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "helpText": lambda n : setattr(self, 'help_text', n.get_str_value()),
            "key": lambda n : setattr(self, 'key', n.get_str_value()),
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
            "placeholder": lambda n : setattr(self, 'placeholder', n.get_str_value()),
            "required": lambda n : setattr(self, 'required', n.get_bool_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("helpText", self.help_text)
        writer.write_str_value("key", self.key)
        writer.write_str_value("label", self.label)
        writer.write_str_value("placeholder", self.placeholder)
        writer.write_bool_value("required", self.required)
        writer.write_str_value("type", self.type)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .....models.i_result import IResult
    from .....models.update_webhook_request import UpdateWebhookRequest
    from .....models.webhook_operation_response import WebhookOperationResponse
    from .deactivate.deactivate_request_builder import DeactivateRequestBuilder
    from .deliveries.deliveries_request_builder import DeliveriesRequestBuilder
    from .link_provider.link_provider_request_builder import LinkProviderRequestBuilder
    from .reactivate.reactivate_request_builder import ReactivateRequestBuilder
    from .rotate_secret.rotate_secret_request_builder import RotateSecretRequestBuilder
    from .test.test_request_builder import TestRequestBuilder

class WebhooksItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api/v1/webhooks/{id}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WebhooksItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api/v1/webhooks/{id}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WebhookOperationResponse]:
        """
        DELETE_api_v1_webhooks_id
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WebhookOperationResponse]
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.webhook_operation_response import WebhookOperationResponse

        return await self.request_adapter.send_async(request_info, WebhookOperationResponse, None)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[IResult]:
        """
        GET_api_v1_webhooks_id
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[IResult]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.i_result import IResult

        return await self.request_adapter.send_async(request_info, IResult, None)
    
    async def put(self,body: UpdateWebhookRequest, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WebhookOperationResponse]:
        """
        PUT_api_v1_webhooks_id
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WebhookOperationResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_put_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .....models.webhook_operation_response import WebhookOperationResponse

        return await self.request_adapter.send_async(request_info, WebhookOperationResponse, None)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        DELETE_api_v1_webhooks_id
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        GET_api_v1_webhooks_id
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_put_request_information(self,body: UpdateWebhookRequest, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        PUT_api_v1_webhooks_id
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PUT, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> WebhooksItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WebhooksItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WebhooksItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def deactivate(self) -> DeactivateRequestBuilder:
        """
        The deactivate property
        """
        from .deactivate.deactivate_request_builder import DeactivateRequestBuilder

        return DeactivateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def deliveries(self) -> DeliveriesRequestBuilder:
        """
        The deliveries property
        """
        from .deliveries.deliveries_request_builder import DeliveriesRequestBuilder

        return DeliveriesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def link_provider(self) -> LinkProviderRequestBuilder:
        """
        The linkProvider property
        """
        from .link_provider.link_provider_request_builder import LinkProviderRequestBuilder

        return LinkProviderRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def reactivate(self) -> ReactivateRequestBuilder:
        """
        The reactivate property
        """
        from .reactivate.reactivate_request_builder import ReactivateRequestBuilder

        return ReactivateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rotate_secret(self) -> RotateSecretRequestBuilder:
        """
        The rotateSecret property
        """
        from .rotate_secret.rotate_secret_request_builder import RotateSecretRequestBuilder

        return RotateSecretRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def test(self) -> TestRequestBuilder:
        """
        The test property
        """
        from .test.test_request_builder import TestRequestBuilder

        return TestRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WebhooksItemRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WebhooksItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WebhooksItemRequestBuilderPutRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ...........models.e2_e_encrypted_payload import E2EEncryptedPayload
    from ...........models.operation_response import OperationResponse
    from ...........models.problem_details import ProblemDetails
    from ...........models.update_secret_request import UpdateSecretRequest
    from ...........models.update_secret_response import UpdateSecretResponse
    from .metadata.metadata_request_builder import MetadataRequestBuilder
    from .rollback.rollback_request_builder import RollbackRequestBuilder
    from .rotate.rotate_request_builder import RotateRequestBuilder
    from .rotation_policy.rotation_policy_request_builder import RotationPolicyRequestBuilder
    from .versions.versions_request_builder import VersionsRequestBuilder

class WithKeyItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api/v1/projects/{-id}/environments/{envSlug}/providers/{providerSlug}/secrets/{key}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithKeyItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api/v1/projects/{%2Did}/environments/{envSlug}/providers/{providerSlug}/secrets/{key}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[OperationResponse]:
        """
        DELETE_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[OperationResponse]
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        from ...........models.problem_details import ProblemDetails

        error_mapping: dict[str, type[ParsableFactory]] = {
            "404": ProblemDetails,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...........models.operation_response import OperationResponse

        return await self.request_adapter.send_async(request_info, OperationResponse, error_mapping)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[E2EEncryptedPayload]:
        """
        GET_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[E2EEncryptedPayload]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ...........models.problem_details import ProblemDetails

        error_mapping: dict[str, type[ParsableFactory]] = {
            "400": ProblemDetails,
            "404": ProblemDetails,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...........models.e2_e_encrypted_payload import E2EEncryptedPayload

        return await self.request_adapter.send_async(request_info, E2EEncryptedPayload, error_mapping)
    
    async def put(self,body: UpdateSecretRequest, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[UpdateSecretResponse]:
        """
        PUT_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[UpdateSecretResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_put_request_information(
            body, request_configuration
        )
        from ...........models.problem_details import ProblemDetails

        error_mapping: dict[str, type[ParsableFactory]] = {
            "400": ProblemDetails,
            "404": ProblemDetails,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...........models.update_secret_response import UpdateSecretResponse

        return await self.request_adapter.send_async(request_info, UpdateSecretResponse, error_mapping)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        DELETE_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        GET_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_put_request_information(self,body: UpdateSecretRequest, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        PUT_api_v1_projects_projectRef_environments_envSlug_providers_providerSlug_secrets_key
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PUT, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> WithKeyItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithKeyItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithKeyItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def metadata(self) -> MetadataRequestBuilder:
        """
        The metadata property
        """
        from .metadata.metadata_request_builder import MetadataRequestBuilder

        return MetadataRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rollback(self) -> RollbackRequestBuilder:
        """
        The rollback property
        """
        from .rollback.rollback_request_builder import RollbackRequestBuilder

        return RollbackRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rotate(self) -> RotateRequestBuilder:
        """
        The rotate property
        """
        from .rotate.rotate_request_builder import RotateRequestBuilder

        return RotateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rotation_policy(self) -> RotationPolicyRequestBuilder:
        """
        The rotationPolicy property
        """
        from .rotation_policy.rotation_policy_request_builder import RotationPolicyRequestBuilder

        return RotationPolicyRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions(self) -> VersionsRequestBuilder:
        """
        The versions property
        """
        from .versions.versions_request_builder import VersionsRequestBuilder

        return VersionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WithKeyItemRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithKeyItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithKeyItemRequestBuilderPutRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class CopyEnvironmentCommand(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The copyProviders property
    copy_providers: Optional[bool] = None
    # The copySecretKeys property
    copy_secret_keys: Optional[bool] = None
    # The description property
    description: Optional[str] = None
    # The name property
    name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CopyEnvironmentCommand:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CopyEnvironmentCommand
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CopyEnvironmentCommand()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "copyProviders": lambda n : setattr(self, 'copy_providers', n.get_bool_value()),
            "copySecretKeys": lambda n : setattr(self, 'copy_secret_keys', n.get_bool_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("copyProviders", self.copy_providers)
        writer.write_bool_value("copySecretKeys", self.copy_secret_keys)
        writer.write_str_value("description", self.description)
        writer.write_str_value("name", self.name)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .ca.ca_request_builder import CaRequestBuilder
    from .certificates.certificates_request_builder import CertificatesRequestBuilder
    from .issue.issue_request_builder import IssueRequestBuilder
    from .revoke.revoke_request_builder import RevokeRequestBuilder
    from .roles.roles_request_builder import RolesRequestBuilder
    from .tidy.tidy_request_builder import TidyRequestBuilder

class PkiRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api/v1/projects/{-id}/environments/{envSlug}/pki
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PkiRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api/v1/projects/{%2Did}/environments/{envSlug}/pki", path_parameters)
    
    @property
    def ca(self) -> CaRequestBuilder:
        """
        The ca property
        """
        from .ca.ca_request_builder import CaRequestBuilder

        return CaRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def certificates(self) -> CertificatesRequestBuilder:
        """
        The certificates property
        """
        from .certificates.certificates_request_builder import CertificatesRequestBuilder

        return CertificatesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issue(self) -> IssueRequestBuilder:
        """
        The issue property
        """
        from .issue.issue_request_builder import IssueRequestBuilder

        return IssueRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def revoke(self) -> RevokeRequestBuilder:
        """
        The revoke property
        """
        from .revoke.revoke_request_builder import RevokeRequestBuilder

        return RevokeRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def roles(self) -> RolesRequestBuilder:
        """
        The roles property
        """
        from .roles.roles_request_builder import RolesRequestBuilder

        return RolesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def tidy(self) -> TidyRequestBuilder:
        """
        The tidy property
        """
        from .tidy.tidy_request_builder import TidyRequestBuilder

        return TidyRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .tenant_status import TenantStatus

@dataclass
class GetTenantResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The createdAt property
    created_at: Optional[datetime.datetime] = None
    # The domain property
    domain: Optional[str] = None
    # The name property
    name: Optional[str] = None
    # The slug property
    slug: Optional[str] = None
    # The status property
    status: Optional[TenantStatus] = None
    # The tenantId property
    tenant_id: Optional[UUID] = None
    # The updatedAt property
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> GetTenantResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: GetTenantResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return GetTenantResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .tenant_status import TenantStatus

        from .tenant_status import TenantStatus

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "domain": lambda n : setattr(self, 'domain', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "slug": lambda n : setattr(self, 'slug', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(TenantStatus)),
            "tenantId": lambda n : setattr(self, 'tenant_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("domain", self.domain)
        writer.write_str_value("name", self.name)
        writer.write_str_value("slug", self.slug)
        writer.write_enum_value("status", self.status)
        writer.write_uuid_value("tenantId", self.tenant_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_additional_data_value(self.additional_data)
    

